﻿using Schedule.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using Xamarin.Forms;
using Syncfusion.SfSchedule.XForms;
using System.ComponentModel;

namespace Schedule
{
    public class ScheduleViewModel
    {
        private List<string> currentDayMeetings;

        /// <summary>
        /// color collection
        /// </summary>
        private List<Color> colorCollection;

        /// <summary>
        /// list of meeting
        /// </summary>
        private ObservableCollection<Appointments> events;

        /// <summary>
        /// name collection
        /// </summary>

        internal List<string> nameCollection;

        /// <summary>
        /// resources
        /// </summary>
        private ObservableCollection<object> employees;

        /// <summary>
        /// selected Resources
        /// </summary>
        private ObservableCollection<object> selectedResources;
        public ScheduleViewModel()
        {
            this.events = new ObservableCollection<Appointments>();
            employees = new ObservableCollection<object>();
            selectedResources = new ObservableCollection<object>();
            this.InitializeDataForBookings();
            this.InitializeResources();
            this.BookingAppointments(); 

        }

        private bool showResourceView = true;

        public bool ShowResourceView
        {
            get { return showResourceView; }
            set
            {
                showResourceView = value;
                this.RaiseOnPropertyChanged("ShowResourceView");
            }
        }

        /// <summary>
        /// Property changed event handler
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #region ListOfMeeting

        /// <summary>
        /// Gets or sets list of meeting
        /// </summary>
        public ObservableCollection<Appointments> Appointments
        {
            get
            {
                return this.events;
            }

            set
            {
                this.events = value;
                this.RaiseOnPropertyChanged("Events");
            }
        }
        #endregion

        public ObservableCollection<object> Resources
        {
            get
            {
                return employees;
            }

            set
            {
                employees = value;
                this.RaiseOnPropertyChanged("Employees");
            }
        }


        public ObservableCollection<object> SelectedResources
        {
            get
            {
                return selectedResources;
            }

            set
            {
                selectedResources = value;
                this.RaiseOnPropertyChanged("SelectedEmployees");
            }
        }

        private void InitializeResources()
        {
            Random random = new Random();
            for (int i = 0; i < 6; i++)
            {
                Resources.Add(new Resources()
                {
                    Name = nameCollection[random.Next(0, 5)],
                    Color = Color.FromRgb(random.Next(0, 255), random.Next(10, 255), random.Next(100, 255)),
                    ID = "560" + i.ToString(),
                });
            }

            for (int i = 0; i < 6; i++)
            {
                this.selectedResources.Add(Resources[random.Next(Resources.Count)]);
            }
        }

        #region BookingAppointments

        /// <summary>
        /// Method for booking appointments.
        /// </summary>
        private void BookingAppointments()
        {

            Appointments.Clear();
            Random randomTime = new Random();
            Random randomService = new Random();
            List<Point> randomTimeCollection = new List<Point>();
            randomTimeCollection.Add(new Point(9, 10));
            randomTimeCollection.Add(new Point(10, 11));
            randomTimeCollection.Add(new Point(11, 12));
            randomTimeCollection.Add(new Point(12, 14));
            randomTimeCollection.Add(new Point(14, 16));
            randomTimeCollection.Add(new Point(16, 18));

            var duration = new List<double> { 0.5, 1, 1.5, 2 };
            DateTime date;
            DateTime dateFrom = DateTime.Now;
            DateTime dateTo = DateTime.Now.AddDays(40);
            DateTime dateRangeStart = DateTime.Now;
            DateTime dateRangeEnd = DateTime.Now.AddDays(40);

            var services = new List<string> { "Activity 1", "Activity 2", "Activity 3" };

            foreach (var resource in Resources)
            {
                for (date = dateFrom; date < dateTo; date = date.AddDays(1))
                {
                    var randomCount = randomTime.Next(2, 5);
                    for (int additionalAppointmentIndex = 0; additionalAppointmentIndex < randomCount; additionalAppointmentIndex++)
                    {
                        var rnd = randomService.Next(0, 2);
                        Appointments appointment = new Appointments();
                        int hour = randomTime.Next((int)randomTimeCollection[additionalAppointmentIndex].X, (int)randomTimeCollection[additionalAppointmentIndex].Y);
                        appointment.From = new DateTime(date.Year, date.Month, date.Day, randomTime.Next(5, 20), 0, 0);
                        appointment.To = appointment.From.AddHours(duration[randomTime.Next(0, 3)]);
                        appointment.Color = colorCollection[rnd];
                        appointment.ServiceName = services[rnd];
                        var coll = new ObservableCollection<object>
                            {
                                (resource as Resources).ID
                            };
                        appointment.Resources = coll;

                        Appointments.Add(appointment);
                    }

                }
            }

        }

        #endregion BookingAppointments

        #region InitializeDataForBookings

        /// <summary>
        /// Method for initialize data bookings.
        /// </summary>
        private void InitializeDataForBookings()
        {
            this.currentDayMeetings = new List<string>();
            this.currentDayMeetings.Add("General Meeting");
            this.currentDayMeetings.Add("Plan Execution");
            this.currentDayMeetings.Add("Project Plan");
            this.currentDayMeetings.Add("Consulting");
            this.currentDayMeetings.Add("Performance Check");
            this.currentDayMeetings.Add("Yoga Therapy");
            this.currentDayMeetings.Add("Plan Execution");
            this.currentDayMeetings.Add("Project Plan");
            this.currentDayMeetings.Add("Consulting");
            this.currentDayMeetings.Add("Performance Check");

            this.colorCollection = new List<Color>();
            this.colorCollection.Add(Color.FromHex("#FF339933"));
            this.colorCollection.Add(Color.FromHex("#FF00ABA9"));
            this.colorCollection.Add(Color.FromHex("#FFE671B8"));
            this.colorCollection.Add(Color.FromHex("#FF1BA1E2"));
            this.colorCollection.Add(Color.FromHex("#FFD80073"));
            this.colorCollection.Add(Color.FromHex("#FFA2C139"));
            this.colorCollection.Add(Color.FromHex("#FFA2C139"));
            this.colorCollection.Add(Color.FromHex("#FFD80073"));
            this.colorCollection.Add(Color.FromHex("#FF339933"));
            this.colorCollection.Add(Color.FromHex("#FFE671B8"));
            this.colorCollection.Add(Color.FromHex("#FF00ABA9"));

            this.nameCollection = new List<string>();
            this.nameCollection.Add("Brooklyn");
            this.nameCollection.Add("James William");
            this.nameCollection.Add("Sophia");
            this.nameCollection.Add("Stephen");
            this.nameCollection.Add("Zoey Addison");
            this.nameCollection.Add("Daniel");
        }

        #endregion InitializeDataForBookings

        #region Property Changed Event

        /// <summary>
        /// Invoke method when property changed
        /// </summary>
        /// <param name="propertyName">property name</param>
        private void RaiseOnPropertyChanged(string propertyName)
        {
            this.PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}
